module Livermore
  class Bot
    CONFIG_PATH = 'config.yml.erb'.freeze

    attr_reader :market, :best_ask, :best_bid, :base_fund, :quote_fund
    attr_accessor :ask, :bid, :open_orders, :orders_to_create, :orders_to_cancel

    def initialize(config_path = CONFIG_PATH)
      config = load_config(config_path)

      initialize_exchange_client(config)
      initialize_instance_variables(config)
      initialize_strategy(config)
    end

    def trade
      @strategy.trade
    end

    def fetch_top_tier
      response = orderbook(@market)

      @best_ask = response.best_ask
      @best_bid = response.best_bid
    end

    def fetch_orders
      response = my_orders(market: @market)

      @open_orders = response.open_orders
    end

    def cancel_orders
      orders_to_cancel.each do |order|
        response = cancel_order(order.id)

        update_fund(:+, order) if response.code == 200
      end
    end

    def create_orders
      orders_to_create.each do |order|
        response = create_order(@order_klass.new(order).as_json)

        update_fund(:-, order) if response.code == 201
      end

      orders_to_create.clear
    end

    def cancel_all
      fetch_orders
      @orders_to_cancel = open_orders
      cancel_orders
    end

    def update_fund(method, order)
      case order.type
      when Order::ASK
        @base_fund = @base_fund.send(method, order.amount)
      when Order::BID
        @quote_fund = @quote_fund.send(method, order.amount * order.price)
      end
    end

    def method_missing(method, *args)
      if @client.respond_to?(method)
        @response_klass.new(@client.send(method, *args))
      else
        super
      end
    end

    private

    def load_config(path)
      template = ERB.new(File.read(path))

      YAML.load(template.result)
    end

    def initialize_exchange_client(config)
      name = config[:exchange][:name]
      key = config[:exchange][:key]
      secret = config[:exchange][:secret]

      @client = CLIENTS[name][:client].new(key, secret)
      @response_klass = CLIENTS[name][:response]
      @order_klass = CLIENTS[name][:order]
    end

    def initialize_instance_variables(config)
      @market = config[:market]
      @base_fund = config[:base_fund]
      @quote_fund = config[:quote_fund]
      @orders_to_create = []
    end

    def initialize_strategy(config)
      name = config[:strategy][:name]
      path = config[:strategy][:path]

      @strategy = STRATEGIES[name].new(self, path)
    end
  end
end
